#pragma once

#include <cstdio>
#include <cassert>
#include <ctime>
#include <sstream>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <vector>
#include <random>
#include <boost/lexical_cast.hpp>

typedef   signed char        sint1;
typedef unsigned char        uint1;
typedef   signed short       sint2;
typedef unsigned short       uint2;
typedef   signed int         sint4;
typedef unsigned int         uint4;
typedef               float  real4;
typedef               double real8;
typedef          long double realC;
typedef   signed long long   sint8;
typedef unsigned long long   uint8;

typedef void* vptr;
typedef char* cptr;

typedef const void* cvptr;
typedef const char* ccptr;

typedef const void* const cvptrc;
typedef const char* const ccptrc;

const sint1 min_sint1 = 0x80;
const sint1 max_sint1 = 0x7F;
const uint1 min_uint1 = 0x00;
const uint1 max_uint1 = 0xFF;
const sint2 min_sint2 = 0x8000;
const sint2 max_sint2 = 0x7FFF;
const uint2 min_uint2 = 0x0000;
const uint2 max_uint2 = 0xFFFF;
const sint4 min_sint4 = 0x80000000;
const sint4 max_sint4 = 0x7FFFFFFF;
const uint4 min_uint4 = 0x00000000;
const uint4 max_uint4 = 0xFFFFFFFF;
const sint8 min_sint8 = 0x8000000000000000LL;
const sint8 max_sint8 = 0x7FFFFFFFFFFFFFFFLL;
const uint8 min_uint8 = 0x0000000000000000LL;
const uint8 max_uint8 = 0xFFFFFFFFFFFFFFFFLL;

#define lc boost::lexical_cast

typedef std::mt19937 RNG;

#define FOREVER    for (;;)

// frequently used for loops
// ..._DYN evaluates upper bound in each loop
// ... evaluates it only once at the loop start

// important: 2 statements => put { } around FOR in if/else/while/for blocks
#define FOR(i, n)      auto _=(n); for (i=0; i < _; ++i)
#define FOR_DYN(i, n)  for (i=0; i < (n); ++i)

#define FORS(i, n) for (sint4 i=0, _=(n); i < _; ++i)
#define FORS_DYN(i, n) for (sint4 i=0; i < (n); ++i)

#define FORU(i, n) for (uint4 i=0, _=(n); i < _; ++i)
#define FORU_DYN(i, n) for (uint4 i=0; i < (n); ++i)

#define FORS_REV(i, n) for (sint4 i=(n)-1; i >= 0; --i)

// infers the upper bound type
#define FORT(i, n)      for (decltype(n) i=0, _=(n); i < _; ++i)
#define FORT_DYN(i, n)  for (decltype(n) i=0; i < (n); ++i)

// these are macros that traverse STL containers

// assumes container does not change
#define FORALL(CONT,i)  \
  for (auto i = (CONT).begin(), _ = (CONT).end(); i != _; ++i)

// assumes container does not change and defines iterator outside of loop (B=BREAK)
// ATTENTION: two lines of code => { } may be necessary!
#define FORALL_B(CONT,i) \
  auto i = (CONT).begin(); \
  for (auto _ = (CONT).end(); i != _; ++i) \

// container may change
#define FORALL_DYN(CONT,i)      \
  for (auto i = (CONT).begin(); i != (CONT).end(); ++i)

// reverse traversal
// container does not change
#define FORALL_REV(CONT,i)      \
  for (auto i = (CONT).rbegin(), _ = (CONT).rend(); i != _; ++i)

// reverse traversal
// assumes container does not change (BREAK version)
// ATTENTION: two lines of code => { } may be necessary!
#define FORALL_REV_B(CONT,i)    \
  auto i = (CONT).rbegin(); \
  for (auto _ = (CONT).rend(); i != _; ++i)

// container may change
#define FORALL_REV_DYN(CONT,i)  \
  for (auto i = (CONT).rbegin(); i != (CONT).rend(); ++i)

#define FIND(cont, it, elem) auto it = cont.find(elem);

#define STR2STRING(os, s) \
{\
  (s).erase(); (s).append((os).str(), (os).pcount());\
  (os).rdbuf()->freeze(false);\
} 


//inline int round(double x) { return int(rint(x)); }

std::ostream& Form( std::ostream& os, const char* fmt, ... );

#if defined(__GNUC__)

#define ERR(s) {\
    std::cerr << std::endl << __FILE__ << " " << __FUNCTION__ << "() (line " << __LINE__ << "): " \
              << s << std::endl; ::abort();                             \
}

#define ERR2(s1, s2) {\
    std::cerr << std::endl << __FILE__ << " " << __FUNCTION__ << "() (line " << __LINE__ << "): " \
              << s1 << " " << s2 << std::endl; ::abort();               \
}

#else



#define ERR(s) {\
  cerr << endl << __FILE__  << " (line " << __LINE__ << "): "\
  << s << endl; ::abort();\
}

#define ERR2(s1, s2) {\
  cerr << endl << __FILE__  << " (line " << __LINE__ << "): "\
  << s1 << " " << s2 << endl; ::abort();\
}
#endif

