#pragma once

#include "CPP.H"
#include <mpd/client.h>
#include <boost/filesystem.hpp>
#include "MPD.H"
#include "PlayMode.H"
#include "PlayNode.H"

class Interface
{
public:

  // sent to web clients for adjusting their interface
  enum Mode { AUDIO=0, VIDEO, BLU, WII, CD_CLASSIC, CD_POP, OFF };
  
  Interface(const std::string &music_dir,
            std::ostream &vfd_os_,
            mpd_connection *mpd_conn,
            RNG &rng,
            int update_hz,
            Mode initial_mode,
            int play_mode);
  
  // actions
  void action_select();
  void action_cw();
  void action_ccw();
  void action_return();
  
  void action_previous();
  void action_next();
  void action_seek_backward();
  void action_seek_forward();  
  void action_stop();
  void action_playpause();

  void action_mode();
  
  void action_toggle_shuffle();

  // update status (called regularly)
  void update(int tick);

  std::string status() const;
  void set_mode(Mode m) { mode = m; }
  
private:

  bool on_tick(int secs);
  void vfd(const std::string &msg);
  void vfd_update();
  void new_work_node(PlayNode *node, int new_item);
  void seek(int secs);
  void scan_files();
  
  // state
  bool shuffle;
  bool repeat;  
  bool playing;
  bool pausing;
  bool mode2;     // selection: select play mode, playing: rate (not implemented yet)
  PlayNode *play_node_tree = 0;
  int classify_index;
  boost::filesystem::path home_path;
  PlayNode *work_node; // current node in tree
  std::vector<PlayNode*> work_item_nodes;   // folder nodes (marked)
  std::vector<std::string> work_item_strs;  // folder strings
  int current_item; // current item in work_node
  std::vector<int> item_stack; // selection history
  std::vector<PlayNode*> work_node_stack;

  int tick = 0; // passed to update function
  int last_action_tick = 0; // for screen saver
  MPD_Status mpd_status;
  bool mpd_status_updated = false;
  int freeze_ticks = 0; // how long to freeze messages after pressing prev/next
  int delta_tick1=0, delta_tick2=0; // displacements for line1 / line2
  int selection_tick = 0; // displacement for selection
  std::vector<std::string> songs; // sent to MPD
  int work_path_start_index = 0;
  
  // environment
  std::ostream &vfd_os; // vfs or cout
  mpd_connection *mpd_conn; // =0: not available
  RNG &rng;
  int update_hz; // how often update is called per second
  Mode mode;  
  PlayMode play_mode;

  const int PAUSE_SECS  =  1;
  const int SWAP_SECS   = 60;
  const int FREEZE_SECS =  2;  // prev/next
  const int SAVER_SECS  = 60;  // ;-)
  const int SEEK_BACKWARD_SECS = -10;
  const int SEEK_FORWARD_SECS  = 30;

  static std::vector<std::string> mode_names;
};
