#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <csignal>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cmath>
#include <boost/format.hpp>
#include "MP3Info.H"

using namespace std;

#define MAX_STR_LEN 100
#define GENRE_MAX 0x94

#define real float

/*  #define         MAX_NAME_SIZE           81 */
#define         SBLIMIT                 32
#define         SCALE_BLOCK             12
#define         SSLIMIT                 18

#define         MPG_MD_STEREO           0
#define         MPG_MD_JOINT_STEREO     1
#define         MPG_MD_DUAL_CHANNEL     2
#define         MPG_MD_MONO             3

struct id3v1tag_t {
        char tag[3]; /* always "TAG": defines ID3v1 tag 128 bytes before EOF */
        char title[30];
        char artist[30];
        char album[30];
        char year[4];
        char comment[30];
        unsigned char genre;
};

struct id3tag_t {
        char title[64];
        char artist[64];
        char album[64];
        char year[5];
        char comment[256];
        char genre[256];
};

struct al_table
{
        short bits;
        short d;
};

struct frame
{
        struct al_table *alloc;
        int (*synth) (real *, int, unsigned char *, int *);
        int (*synth_mono) (real *, unsigned char *, int *);
        int stereo;
        int jsbound;
        int single;
        int II_sblimit;
        int down_sample_sblimit;
        int lsf;
        int mpeg25;
        int down_sample;
        int header_change;
        int lay;
        int (*do_layer) (struct frame * fr);
        int error_protection;
        int bitrate_index;
        int sampling_frequency;
        int padding;
        int extension;
        int mode;
        int mode_ext;
        int copyright;
        int original;
        int emphasis;
        int framesize;          /* computed framesize */
};


/* max = 1728 */
#define MAXFRAMESIZE 1792

static int tabsel_123[2][3][16] =
{
        {
    {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448,},
       {0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384,},
       {0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320,}},

        {
       {0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256,},
            {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160,},
            {0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160,}}
};

static int ssize;

long mpg123_freqs[9] =
{44100, 48000, 32000, 22050, 24000, 16000, 11025, 12000, 8000};

extern int mpg123_bitrate, mpg123_frequency, mpg123_length;
extern char *mpg123_title, *mpg123_filename;
extern bool mpg123_stereo;

// static int fsizeold = 0, ssize;
// static unsigned char bsspace[2][MAXFRAMESIZE + 512];    /* MAXFRAMESIZE */
// static unsigned char *bsbuf = bsspace[1], *bsbufold;
// static int bsnum = 0;

unsigned char *mpg123_pcm_sample;

#define N_(s) (s)

const char *MP3Info::emphasis_txt[] = {
  N_("None"),
  N_("50/15 ms"),
  "",
  N_("CCIT J.17")
};

const char *MP3Info::channel_mode_txt[] = {
  N_("Stereo"),
  N_("Joint stereo"),
  N_("Dual channel"),
  N_("Single channel")
};

const char *MP3Info::genre_txt[GENRE_MAX] =
{
  N_("Blues"), N_("Classic Rock"), N_("Country"), N_("Dance"),
  N_("Disco"), N_("Funk"), N_("Grunge"), N_("Hip-Hop"),
  N_("Jazz"), N_("Metal"), N_("New Age"), N_("Oldies"),
  N_("Other"), N_("Pop"), N_("R&B"), N_("Rap"), N_("Reggae"),
  N_("Rock"), N_("Techno"), N_("Industrial"), N_("Alternative"),
  N_("Ska"), N_("Death Metal"), N_("Pranks"), N_("Soundtrack"),
  N_("Euro-Techno"), N_("Ambient"), N_("Trip-Hop"), N_("Vocal"),
  N_("Jazz+Funk"), N_("Fusion"), N_("Trance"), N_("Classical"),
  N_("Instrumental"), N_("Acid"), N_("House"), N_("Game"),
  N_("Sound Clip"), N_("Gospel"), N_("Noise"), N_("Alt"),
  N_("Bass"), N_("Soul"), N_("Punk"), N_("Space"),
  N_("Meditative"), N_("Instrumental Pop"),
  N_("Instrumental Rock"), N_("Ethnic"), N_("Gothic"),
  N_("Darkwave"), N_("Techno-Industrial"), N_("Electronic"),
  N_("Pop-Folk"), N_("Eurodance"), N_("Dream"),
  N_("Southern Rock"), N_("Comedy"), N_("Cult"),
  N_("Gangsta Rap"), N_("Top 40"), N_("Christian Rap"),
  N_("Pop/Funk"), N_("Jungle"), N_("Native American"),
  N_("Cabaret"), N_("New Wave"), N_("Psychedelic"), N_("Rave"),
  N_("Showtunes"), N_("Trailer"), N_("Lo-Fi"), N_("Tribal"),
  N_("Acid Punk"), N_("Acid Jazz"), N_("Polka"), N_("Retro"),
  N_("Musical"), N_("Rock & Roll"), N_("Hard Rock"), N_("Folk"),
  N_("Folk/Rock"), N_("National Folk"), N_("Swing"),
  N_("Fast-Fusion"), N_("Bebob"), N_("Latin"), N_("Revival"),
  N_("Celtic"), N_("Bluegrass"), N_("Avantgarde"),
  N_("Gothic Rock"), N_("Progressive Rock"),
  N_("Psychedelic Rock"), N_("Symphonic Rock"), N_("Slow Rock"),
  N_("Big Band"), N_("Chorus"), N_("Easy Listening"),
  N_("Acoustic"), N_("Humour"), N_("Speech"), N_("Chanson"),
  N_("Opera"), N_("Chamber Music"), N_("Sonata"), N_("Symphony"),
  N_("Booty Bass"), N_("Primus"), N_("Porn Groove"),
  N_("Satire"), N_("Slow Jam"), N_("Club"), N_("Tango"),
  N_("Samba"), N_("Folklore"), N_("Ballad"), N_("Power Ballad"),
  N_("Rhythmic Soul"), N_("Freestyle"), N_("Duet"),
  N_("Punk Rock"), N_("Drum Solo"), N_("A Cappella"),
  N_("Euro-House"), N_("Dance Hall"), N_("Goa"),
  N_("Drum & Bass"), N_("Club-House"), N_("Hardcore"),
  N_("Terror"), N_("Indie"), N_("BritPop"), N_("Negerpunk"),
  N_("Polsk Punk"), N_("Beat"), N_("Christian Gangsta Rap"),
  N_("Heavy Metal"), N_("Black Metal"), N_("Crossover"),
  N_("Contemporary Christian"), N_("Christian Rock"),
  N_("Merengue"), N_("Salsa"), N_("Thrash Metal"),
  N_("Anime"), N_("JPop"), N_("Synthpop")
};



// --------------------------------------------------------------------


static bool mpg123_head_check(unsigned long head)
{
  if ((head & 0xffe00000) != 0xffe00000)
    return false;
  if (!((head >> 17) & 3))
    return false;
  if (((head >> 12) & 0xf) == 0xf)
    return false;
  if (!((head >> 12) & 0xf))
    return false;
  if (((head >> 10) & 0x3) == 0x3)
    return false;
  if (((head >> 19) & 1) == 1 && ((head >> 17) & 3) == 3 && ((head >> 16) & 1) == 1)
    return false;
  if ((head & 0xffff0000) == 0xfffe0000)
    return false;
        
  return true;
}

/*
 * the code a header and write the information
 * into the frame structure
 */
static int mpg123_decode_header(struct frame *fr, unsigned long newhead)
{
  if (newhead & (1 << 20))
    {
      fr->lsf = (newhead & (1 << 19)) ? 0x0 : 0x1;
      fr->mpeg25 = 0;
    }
  else
    {
      fr->lsf = 1;
      fr->mpeg25 = 1;
    }
  fr->lay = 4 - ((newhead >> 17) & 3);
  if (fr->mpeg25)
    {
      fr->sampling_frequency = 6 + ((newhead >> 10) & 0x3);
    }
  else
    fr->sampling_frequency = ((newhead >> 10) & 0x3) + (fr->lsf * 3);
  fr->error_protection = ((newhead >> 16) & 0x1) ^ 0x1;

  if (fr->mpeg25)         /* allow Bitrate change for 2.5 ... */
    fr->bitrate_index = ((newhead >> 12) & 0xf);

  fr->bitrate_index = ((newhead >> 12) & 0xf);
  fr->padding = ((newhead >> 9) & 0x1);
  fr->extension = ((newhead >> 8) & 0x1);
  fr->mode = ((newhead >> 6) & 0x3);
  fr->mode_ext = ((newhead >> 4) & 0x3);
  fr->copyright = ((newhead >> 3) & 0x1);
  fr->original = ((newhead >> 2) & 0x1);
  fr->emphasis = newhead & 0x3;

  fr->stereo = (fr->mode == MPG_MD_MONO) ? 1 : 2;

  if (!fr->bitrate_index) return (0);

  switch (fr->lay) {
  case 1:
    //!!!fr->do_layer = mpg123_do_layer1;
    //!!! mpg123_init_layer2();   /* inits also shared tables with layer1 </*/
    fr->jsbound = (fr->mode == MPG_MD_JOINT_STEREO) ?
      (fr->mode_ext << 2) + 4 : 32;
    fr->framesize = (long) tabsel_123[fr->lsf][0][fr->bitrate_index] * 12000;
    fr->framesize /= mpg123_freqs[fr->sampling_frequency];
    fr->framesize = ((fr->framesize + fr->padding) << 2) - 4;
    break;
  case 2:
    //!!!fr->do_layer = mpg123_do_layer2;
    //!!! mpg123_init_layer2();   /* inits also shared tables with layer1 */
    //!!!get_II_stuff(fr);
    fr->jsbound = (fr->mode == MPG_MD_JOINT_STEREO) ?
      (fr->mode_ext << 2) + 4 : fr->II_sblimit;
    fr->framesize = (long) tabsel_123[fr->lsf][1][fr->bitrate_index] * 144000;
    fr->framesize /= mpg123_freqs[fr->sampling_frequency];
    fr->framesize += fr->padding - 4;
    break;
  case 3:
    //!!!fr->do_layer = mpg123_do_layer3;
    if (fr->lsf)
      ssize = (fr->stereo == 1) ? 9 : 17;
    else
      ssize = (fr->stereo == 1) ? 17 : 32;
    if (fr->error_protection)
      ssize += 2;
    fr->framesize = (long) tabsel_123[fr->lsf][2][fr->bitrate_index] * 144000;
    fr->framesize /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
    fr->framesize = fr->framesize + fr->padding - 4;
    break;
  default:
    return (0);
  }

  if(fr->framesize > MAXFRAMESIZE) return 0;
  return 1;
}

static double mpg123_compute_tpf(struct frame *fr)
{
  static int bs[4] =
  {0, 384, 1152, 1152};
  double tpf;

  tpf = (double) bs[fr->lay];
  tpf /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
  return tpf;
}

static double mpg123_compute_bpf(struct frame *fr)
{
  double bpf;

  switch (fr->lay)
    {
    case 1:
      bpf = tabsel_123[fr->lsf][0][fr->bitrate_index];
      bpf *= 12000.0 * 4.0;
      bpf /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
      break;
    case 2:
    case 3:
      bpf = tabsel_123[fr->lsf][fr->lay - 1][fr->bitrate_index];
      bpf *= 144000;
      bpf /= mpg123_freqs[fr->sampling_frequency] << (fr->lsf);
      break;
    default:
      bpf = 1.0;
    }

  return bpf;
}


/*---- DXhead.c --------------------------------------------


decoder MPEG Layer III

handle Xing header

mod 12/7/98 add vbr scale

Copyright 1998 Xing Technology Corp.
-----------------------------------------------------------*/  

#include <stdlib.h>
#include <stdio.h>
#include <float.h>
#include <math.h>

/* A Xing header may be present in the ancillary
 * data field of the first frame of an mp3 bitstream
 * The Xing header (optionally) contains
 *      frames      total number of audio frames in the bitstream
 *      bytes       total number of bytes in the bitstream
 *      toc         table of contents
 
 * toc (table of contents) gives seek points
 * for random access
 * the ith entry determines the seek point for
 * i-percent duration
 * seek point in bytes = (toc[i]/256.0) * total_bitstream_bytes
 * e.g. half duration seek point = (toc[50]/256.0) * total_bitstream_bytes
 */ 

#define FRAMES_FLAG     0x0001
#define BYTES_FLAG      0x0002
#define TOC_FLAG        0x0004
#define VBR_SCALE_FLAG  0x0008

#define FRAMES_AND_BYTES (FRAMES_FLAG | BYTES_FLAG)

/* structure to receive extracted header
 * toc may be NULL
 */ 
struct XHEADDATA
{
  int h_id;               /* from MPEG header, 0=MPEG2, 1=MPEG1 */
  int samprate;           /* determined from MPEG header */
  int flags;              /* from Xing header data */
  int frames;             /* total bit stream frames from Xing header data */
  int bytes;              /* total bit stream bytes from Xing header data */
  int vbr_scale;          /* encoded vbr scale from Xing header data */
  unsigned char *toc;     /* pointer to unsigned char toc_buffer[100] */
  /* may be NULL if toc not desired */

  XHEADDATA()
  {
    // initialize with 0
    memset(this, 0, sizeof(*this));
  }
};




/* 4   Xing
 * 4   flags
 * 4   frames
 * 4   bytes
 * 100 toc
 */ 

/*-------------------------------------------------------------*/ 

static int ExtractI4(unsigned char *buf) 
{
  int x;
	
  /* big endian extract */ 
		
  x = buf[0];
  x <<= 8;
  x |= buf[1];
  x <<= 8;
  x |= buf[2];
  x <<= 8;
  x |= buf[3];
  return x;
}

/*-------------------------------------------------------------*/ 
/* return 0=fail, 1=success
 * X   structure to receive header data (output)
 * buf bitstream input 
 */ 

static int mpg123_get_xing_header(XHEADDATA * X, unsigned char *buf) 
{
  int i, head_flags;
  int h_id, h_mode, h_sr_index;

  static int sr_table[4] = {44100, 48000, 32000, 99999};
	
  /* get Xing header data */ 
		
  X->flags = 0;	/* clear to null incase fail */
		
  /* get selected MPEG header data */ 
  h_id = (buf[1] >> 3) & 1;
  h_sr_index = (buf[2] >> 2) & 3;
  h_mode = (buf[3] >> 6) & 3;
	
  /* determine offset of header */ 
  if (h_id) {			/* mpeg1 */

    if (h_mode != 3)
      buf += (32 + 4);
    else
      buf += (17 + 4);

  } else {			/* mpeg2 */
		
    if (h_mode != 3)
      buf += (17 + 4);
    else
      buf += (9 + 4);
  }
	
		
  if (buf[0] != 'X') return 0;	/* fail */
  if (buf[1] != 'i') return 0;	/* header not found */
  if (buf[2] != 'n') return 0;
  if (buf[3] != 'g') return 0;
	
  buf += 4;
  X->h_id = h_id;
  X->samprate = sr_table[h_sr_index];
	
  if (h_id == 0) X->samprate >>= 1;
	
  head_flags = X->flags = ExtractI4(buf);
  buf += 4;		/* get flags */
	
		
  if (head_flags & FRAMES_FLAG) {
    X->frames = ExtractI4(buf);
    buf += 4;
  }
	
  if (head_flags & BYTES_FLAG) {
    X->bytes = ExtractI4(buf);
    buf += 4;
  }
	
		
  if (head_flags & TOC_FLAG) {
		
    if (X->toc != NULL) {
			
      for (i = 0; i < 100; i++) X->toc[i] = buf[i];
    }
		
    buf += 100;
  }
	
  X->vbr_scale = -1;
	
  if (head_flags & VBR_SCALE_FLAG) {
    X->vbr_scale = ExtractI4(buf);
    buf += 4;
  }
	
  /*if( X->toc != NULL ) {
   *for(i=0;i<100;i++) {
   *    if( (i%10) == 0 ) printf("\n");
   *    printf(" %3d", (int)(X->toc[i]));
   *}
   *}
   */ 
		
  return 1;	/* success */
}


#if 0

/*-------------------------------------------------------------*/ 
/* return seekpoint in bytes (may be at eof if percent=100.0)
 * TOC = table of contents from Xing header
 * file_bytes = number of bytes in mp3 file
 * percent = play time percentage of total playtime. May be
 * fractional (e.g. 87.245)
 */ 

static int mpg123_seek_point(unsigned char TOC[100], int file_bytes, float percent) 
{
	
  /* interpolate in TOC to get file seek point in bytes */ 
  int a, seekpoint;
	
  float fa, fb, fx;
	
  if (percent < 0.0f)
    percent = 0.0f;
	
  if (percent > 100.0f)
    percent = 100.0f;
	
  a = (int) percent;
	
  if (a > 99)
    a = 99;
	
  fa = TOC[a];
	
  if (a < 99) {
    fb = TOC[a + 1];
  } else {
    fb = 256.0f;
  }
	
  fx = fa + (fb - fa) * (percent - a);
  seekpoint = (int) ((1.0f / 256.0f) * fx * file_bytes);
  return seekpoint;
}

#endif


/*-------------------------------------------------------------*/ 


static void make_string(char *s, int n, string &string)
{
  string.erase();

  for (int i=0; i < n; ++i) string += s[i];
}


// return 0 iff ok
int MP3Info::get(const char *filename)
{
  struct id3v1tag_t tag;
  FILE *fh;

  if ((fh = fopen(filename, "rb")) != NULL) {

    uint4 head;
    unsigned char tmp[4];
    struct frame frm;

    id3_found = false;

    fseek(fh, -sizeof (tag), SEEK_END);
    if (fread(&tag, 1, sizeof (tag), fh) == sizeof (tag)) {
      if (!strncmp(tag.tag, "TAG", 3)) {
	id3_found = true;

	make_string(tag.title, 30, title);
	make_string(tag.artist, 30, artist);
	make_string(tag.album, 30, album);
	make_string(tag.year, 4, year);
	make_string(tag.comment, 30, comment);
	genre = tag.genre;
      }
    }
    
    rewind(fh);

    if (fread(tmp, 1, 4, fh) != 4) { fclose(fh); return 1; }
    
    head =
      ((uint4) tmp[0] << 24) | ((uint4) tmp[1] << 16) |
      ((uint4) tmp[2] << 8) | (uint4) tmp[3];

    while (!mpg123_head_check(head)) {
      head <<= 8;
      if (fread(tmp, 1, 1, fh) != 1) { fclose(fh); return 1; }
      head |= tmp[0];
    }

    if (!mpg123_decode_header(&frm, head)) { fclose(fh); return 1; }

    {
      uint1 *buf;
      double tpf;
      int pos;
      XHEADDATA xing_header;
      buf = (uint1*)malloc(frm.framesize + 4);
      fseek(fh, -4, SEEK_CUR);
      fread(buf, 1, frm.framesize + 4, fh);
      xing_header.toc = NULL;
      tpf = mpg123_compute_tpf(&frm);
      if (frm.mpeg25) level = -1;
      else level = frm.lsf + 1;
      layer = frm.lay;
      pos = ftell(fh);
      fseek(fh, 0, SEEK_END);
      if (mpg123_get_xing_header(&xing_header, buf)) {
	vbr = true;
	num_frames = xing_header.frames;
	bit_rate = ((xing_header.bytes * 8) / (tpf * xing_header.frames * 1000));
      } else {
	vbr = false;
	num_frames = (int) ((ftell(fh) - pos - (id3_found ? 128 : 0)) / mpg123_compute_bpf(&frm)) + 1;
	bit_rate = tabsel_123[frm.lsf][frm.lay - 1][frm.bitrate_index];
      }
  
      sample_rate = mpg123_freqs[frm.sampling_frequency];
      channel_mode = frm.mode;
      error_protection = frm.error_protection;
      copyright = frm.copyright;
      original = frm.original;
      emphasis = frm.emphasis;
      file_size = ftell(fh);
      duration = (file_size * 8.0) / (bit_rate * 1000);
      free(buf);
    }

    fclose(fh);
    return 0;
  }

  return 1;
} 



void MP3Info::print(ostream &os)
{
  if (id3_found) {

    os << "Artist: " << artist << endl;
    os << "Title: "  << title << endl;
    os << "Album: " << album << endl;
    os << "Year: " << year << endl;
    os << "Comment: " << comment << endl;
    os << "Genre: ";
    if (genre > GENRE_MAX) os << endl;
    else os << genre_txt[genre] << endl;

  } else {

    os << "ID3 header not found!" << endl;

  }

  os << "MPEG ";
  if (level < 0) os << "2.5"; else os << level;
  os << " Layer " << layer << endl;

  if (vbr) os << "average ";
  os << "bit-rate: " << bit_rate << endl;
  os << "sample-rate: " << sample_rate << endl;
  os << "frames: " << num_frames << endl;
  os << "channel-mode: " << channel_mode_txt[channel_mode] << endl;
  os << "error-protection: " << error_protection << endl;
  os << "copyright: " << copyright << endl;  
  os << "emphasis: " << emphasis_txt[emphasis] << endl;
  os << "file-size: " << file_size << endl;
  os << "duration: "; os << boost::format("%d:%.1f\n") % int(duration/60) % (duration-int(duration/60)*60);
}
