#include "MPD.H"
#include <cstring>
#include <boost/format.hpp>

using namespace std;
  
MPD_Status::MPD_Status()
{
  memset(this, 0, sizeof(MPD_Status));
}

void MPD_Status::print()
{
  std::cout << boost::format("%d%% r=%d s=%d st=%d e=%d t=%d b=%d id=%d q=%d")
    % volume
    % (repeat ? 1 : 0)
    % (shuffle ? 1: 0)
    % state
    % elapsed_time
    % total_time
    % bit_rate
    % song_pos
    % queue_len;
}

int MPD_Status::update(mpd_connection *mpd_conn)
{
  cout << "MPD update" << endl;
  
  if (!mpd_conn) return 1;
  
  struct mpd_status *mpd_s = mpd_run_status(mpd_conn);
  if (mpd_s == 0) {
    std::cerr << "MPD error: status" << std::endl;
    return 1;
  }

  volume = mpd_status_get_volume(mpd_s);
  repeat = mpd_status_get_repeat(mpd_s);    
  shuffle = mpd_status_get_random(mpd_s);
  state  = mpd_status_get_state(mpd_s);
  elapsed_time = mpd_status_get_elapsed_time(mpd_s);
  total_time = mpd_status_get_total_time(mpd_s);  
  bit_rate = mpd_status_get_kbit_rate(mpd_s);
  song_pos = mpd_status_get_song_pos(mpd_s);
  queue_len = mpd_status_get_queue_length(mpd_s);

  mpd_status_free(mpd_s);
  return 0;
}


int mpd_repeat(mpd_connection *mpd_conn, bool rep)
{
  cout << "MPD command: repeat " << rep << endl;

  if (mpd_conn && !mpd_run_repeat(mpd_conn, rep)) {
    cerr << "MPD error: pause/resume" << endl;
    return 1;
  }
  return 0;
}

int mpd_pause(mpd_connection *mpd_conn, bool pause)
{
  cout << "MPD command: pause " << pause << endl;

  if (mpd_conn && !mpd_run_pause(mpd_conn, pause)) {
    cerr << "MPD error: pause/resume" << endl;
    return 1;
  }
  return 0;
}

int mpd_stop(mpd_connection *mpd_conn)
{
  cout << "MPD command: stop" << endl;

  if (mpd_conn && !mpd_run_stop(mpd_conn)) {
    cerr << "MPD error: stop" << endl;
    return 1;
  }
  return 0;
}

int mpd_play(mpd_connection *mpd_conn)
{
  cout << "MPD command: play" << endl;

  if (mpd_conn && !mpd_run_play(mpd_conn)) {
    cerr << "MPD error: play" << endl;
    return 1;
  }
  return 0;
}

int mpd_clear(mpd_connection *mpd_conn)
{
  cout << "MPD command: clear" << endl;

  if (mpd_conn) {
    mpd_run_clear(mpd_conn);
    if (!mpd_response_finish(mpd_conn)) {
      cerr << "MPD error: clear" << endl;
      return 1;
    }
  }
  return 0;
}

int mpd_previous(mpd_connection *mpd_conn)
{
  cout << "MPD command: previous" << endl;

  if (mpd_conn) {
    mpd_run_previous(mpd_conn);
    if (!mpd_response_finish(mpd_conn)) {
      cerr << "MPD error: previous" << endl;
      return 1;
    }
  }
  return 0;
}

int mpd_seek(mpd_connection *mpd_conn, int song_pos, int time)
{
  cout << "SEEK " << song_pos << " " << time << endl;

  if (mpd_conn) {
    mpd_run_seek_pos(mpd_conn, song_pos, time);
    if (!mpd_response_finish(mpd_conn)) {
      cerr << "MPD error: seek" << endl;
      return 1;
    }
  }
  return 0;
}

int mpd_next(mpd_connection *mpd_conn)
{
  cout << "MPD command: next" << endl;

  if (mpd_conn) {
    mpd_run_next(mpd_conn);
    if (!mpd_response_finish(mpd_conn)) {
      cerr << "MPD error: next" << endl;
      return 1;
    }
  }
  return 0;
}

int mpd_add(mpd_connection *mpd_conn, vector<string> &titles)
{
  cout << "MPD command: add" << endl;

  if (mpd_conn && !mpd_command_list_begin(mpd_conn, false)) {
    cerr << "MPD error: list begin" << endl;
    return 1;
  }
  
  FORALL (titles, it) {
    cout << "MPD: adding " << *it << endl;
    if (mpd_conn)
      mpd_send_add(mpd_conn, it->c_str());
  }

  if (mpd_conn) {
    if (!mpd_command_list_end(mpd_conn)) {
      cerr << "MPD error: list end" << endl;
      return 1;
    }
    
    if (!mpd_response_finish(mpd_conn)) {
      cerr << "MPD error: response finish" << endl;
      return 1;
    }
  }
  return 0;
}


int mpd_update(mpd_connection *mpd_conn)
{
  if (mpd_conn) {
    mpd_run_update(mpd_conn, 0);
    if (!mpd_response_finish(mpd_conn)) {
      cerr << "MPD error: update" << endl;
      return 1;
    }
  }
  return 0;
}

