#pragma once

#include "CPP.H"

class PlayMode
{
public:

  PlayMode(int state) : state(state) {
    if (state < 0 || state >= NUM_STATES)
      ERR("illegal state " << state);
  }
  
  void increment() {
    state = (state + 1) % NUM_STATES;
  }

  void decrement() {
    state = (NUM_STATES + state - 1) % NUM_STATES;
  }

  bool select(int g_val, int l_val) const;

  int get_state() const { return state; }
  
  std::string to_string() const
  {
    return mode_descriptions[state];
  }
  
  enum {
    MODE_GOOD=0,
    MODE_OK,
    MODE_GOOD_LIGHT,
    MODE_GOOD_HEAVY,
    MODE_OK_LIGHT,
    MODE_OK_HEAVY,
    MODE_EXP,
    MODE_ANY,
    MODE_BAD
  };
  
private:

  const char *mode_descriptions[9] = {
    "good",
    "ok+",
    "good & light",
    "good & heavy",
    "ok+ & light",
    "ok+ & heavy",
    "explore",
    "any",
    "bad"
  };

  int state = 0;

public:  
  const int NUM_STATES = sizeof(mode_descriptions)/sizeof(mode_descriptions[0]);
};
