#include "PlayNode.H"
#include "MP3Info.H"
#include "PlayMode.H"
#include <boost/algorithm/string/predicate.hpp>

using namespace std;
using namespace boost::filesystem;

#if 0
static string& pack(string &s)
{
  {
    string::iterator it = s.begin();
    string::iterator hi = s.end();
    for ( ; it != hi && isspace(*it); ++it );
    s.erase( s.begin(), it );
  }
  {
    string::reverse_iterator it = s.rbegin();
    string::reverse_iterator hi = s.rend();
    for ( ; it != hi && isspace(*it); ++it );
    s.erase(s.size() - (it - s.rbegin()), it - s.rbegin() );
  }

  return s;
}
#endif

// creates tree of play nodes

PlayNode *PlayNode::create_tree(const boost::filesystem::path &p,
                                const boost::filesystem::path &home_path)
{
  try {
    if (!exists(p)) return 0; 

    if (is_directory(p)) {
      
      // cout << p << " is a directory containing:\n";
      
      PlayNode *node = new PlayNode;
      if (p != home_path) {
        node->path = p.string().substr(home_path.string().size()+1);
      }
      node->directory = true;
      
      typedef vector<boost::filesystem::path> vec;             // store paths,
      vec v;                                // so we can sort them later

      copy(directory_iterator(p), directory_iterator(), back_inserter(v));

      sort(v.begin(), v.end());             // sort, since directory iteration

      FORALL (v, it) {
        PlayNode *n = create_tree(*it, home_path);
        if (is_regular_file(*it)) {

          string filename = it->filename().string();
          if (isdigit(filename[0])) {
            // mark directory sequential if it contains a file that starts with a digit
            node->random = false;
          }

          if (filename == "g0") {          
            node->g_val = 0;
          } else if (filename == "g1") {
            node->g_val = 1;
          } else if (filename == "g2") {          
            node->g_val = 2;
          } else if (filename == "l0") {          
            node->l_val = 0;
          } else if (filename == "l1") {
            node->l_val = 1;
          }
        }
        
        if (n != 0) {
          node->duration += n->duration;
          node->succ.push_back(n);
        }
      }

      if (!node->succ.empty())
        return node;

      delete node; // nothing playable there
      return 0;
        
    } else if (is_regular_file(p)) {

      string filename = p.string();
      if (filename.substr(filename.size()-4) == ".mp3") {
        MP3Info info;
        if (info.get(filename.c_str())) {
          info.duration = 0;
        }
        // cout << p << " duration: " << info.duration << '\n';
          
        PlayNode *node = new PlayNode;
        node->path = p.string().substr(home_path.string().size()+1);
        node->directory = false;
        node->duration = info.duration;
        return node;
      }
    }
  }

  catch (const filesystem_error& ex) {
    cout << ex.what() << '\n';
  }

  return 0;
}


PlayNode *PlayNode::find(const std::string &search_path) const
{
  if (path == search_path) return (PlayNode*) this; // found

  if (!directory) return 0; // not found

  // cout << path << " == " << search_path << endl;
  
  if (!boost::starts_with(search_path, path)) return 0; // can't be in this branch
  
  FORALL (succ, it) {
    PlayNode *r = (*it)->find(search_path);
    if (r) return r; // found
  }
  return 0; // not found
}

// mark all files that are contained in a selected directory
// and all predecessors of marked directories

bool PlayNode::mark(const PlayMode &play_mode)
{
  marked = false;
  
  if (!directory) return marked;

  bool select = play_mode.select(g_val, l_val);

  FORALL (succ, it) {

    if ((*it)->directory) {

      marked |= (*it)->mark(play_mode);

    } else {

      if (select) {
        (*it)->marked = true;
        marked = true;
      }
    }
  }

  if (marked) cout << path << " " << marked << " " << select << endl;
  
  return marked;
}
