#pragma once

#include "CPP.H"
#include <boost/filesystem.hpp>
#include "PlayMode.H"

// play node created from filesystem traversal

struct PlayNode
{
  std::string path;
  bool directory = false;
  bool random = true;
  int g_val = -1; // good? -1:unknown 0:bad 1:ok 2:good
  int l_val = -1; // light? -1:unknown 0:heavy 1:light
  double duration = 0; // if file
  std::vector<PlayNode*> succ; // if directory
  bool marked = false;
  
  ~PlayNode()
  {
    FORALL (succ, it)
      delete *it;
  }

  // build tree from filesystem node recursively
  static PlayNode *create_tree(const boost::filesystem::path &p,
                               const boost::filesystem::path &home_path);

  // find node in tree by path string
  PlayNode *find(const std::string &search_path) const;

  // mark all files that are contained in a selected directory
  // and all predecessors of marked directories in subtree
  bool mark(const PlayMode &play_mode);
};
