#include "CPP.H"

#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include "Term.H"

using namespace std;

struct termios saved_tty, set_tty;

struct KeyMap {

  int k[5];
  KeyFunc f;
};


KeyMap special_keys[] = {

  { { 27, 91, 65, -1 }, KF_UP },
  { { 27, 91, 66, -1 }, KF_DOWN },
  { { 27, 91, 67, -1 }, KF_RIGHT },
  { { 27, 91, 68, -1 }, KF_LEFT },

  { { 27, 91, 53, 126, -1 }, KF_PAGE_UP },
  { { 27, 91, 54, 126, -1 }, KF_PAGE_DOWN },
  { { 27, 91, 49, 126, -1 }, KF_HOME },
  { { 27, 91, 52, 126, -1 }, KF_END },

  { { 27, 91, 50, 126, -1 }, KF_INS },
  { { 27, 91, 51, 126, -1 }, KF_DEL },

  { { 27, 79, -1 }, KF_SW1 }

};

const int KEYS_NUM = sizeof(special_keys)/sizeof(special_keys[0]);

// unbuffered input, no echo

void set_term()
{
#if 0  
  ioctl(0, TCGETA, &savetty);
  ioctl(0, TCGETA, &settty);
  settty.c_lflag &= ~ICANON;
  settty.c_lflag &= ~ECHO;
  ioctl(0, TCSETA, &settty);
#endif
  
  tcgetattr(0, &saved_tty);
  tcgetattr(0, &set_tty);
  set_tty.c_lflag &= ~(ICANON | ECHO);
  set_tty.c_cc[VTIME] = 0;
  set_tty.c_cc[VMIN] = 1;
  tcsetattr(0, TCSANOW, &set_tty);
}

void reset_term()
{
  tcsetattr(0, TCSANOW, &saved_tty);
}

// read keys: >= 0 char, < 0 function code

int read_char()
{
  enum KeyState { KS_ADD, KS_SW1, KS_WAIT, KS_READY };
  int keys_read = 0;
  KeyState ks = KS_ADD;
  int keys[10];

  int key_func = KF_CHAR;
  int key = 0;

  FOREVER {

    while (ks != KS_READY) {

      int c = getchar();
      // cout << (char)c << ": " << c << endl;

      switch (ks) {

      case KS_ADD:
        {
          keys[keys_read++] = c;

          int i, j;
          FOR (i, KEYS_NUM) {

            FOR (j, keys_read) {
              if (special_keys[i].k[j] != keys[j]) break;
            }

            if (j >= keys_read) break;
          }

          if (i >= KEYS_NUM) {

            // keyseq not found

            if (keys_read == 1) {
              ks = KS_READY;
              key_func = KF_CHAR;
              key = keys[0];

            } else {

              ks = KS_WAIT;

            }


          } else {

            // keyseq found
            if (special_keys[i].k[j] < 0) {

              // complete match

              // cout << "complete match " << j << endl;

              key_func = special_keys[i].f;
              if      (key_func == KF_SW1)  ks = KS_SW1;
              else if (key_func == KF_WAIT) ks = KS_WAIT;
              else {
                ks = KS_READY;
                key_func = key_func;
                key = 0;
              }

            }

            // carry on
          }
          break;
        }
      case KS_SW1:
        ks = KS_ADD;
        keys_read = 0;
        break;

      case KS_WAIT:
        if (c == 126) {
          ks = KS_ADD;
          keys_read = 0;
        }
        break;

      default: ERR("switch error");
      }
    }

    if (key_func == KF_CHAR) {

      return key;

    } else {

      return -key_func;

    }
  }
}
