#pragma once

#include "CPP.H"
#include <queue>
#include <mutex>
#include <map>

struct libwebsocket;
struct libwebsocket_context;

class WebServer
{
public:

  struct Handler {
    virtual void handle(const std::string &msg) = 0;
  };
  
  WebServer(int port);
  ~WebServer();

  // !=0: error
  int run(Handler *handler);
  void broadcast(const std::string &msg);
  
  void shutdown() { done = true; }

  Handler* get_handler() { return handler; }

  // synchronized because queue may change
  void add_client(libwebsocket_context* context, libwebsocket* client);
  void remove_client(libwebsocket* client);
  bool get_msg(libwebsocket* client, std::string &msg);
  
private:

  Handler *handler = 0;
  int port = 0;
  bool done = false;

  // when this size is reached, queues are cleared to catch up with server
  const size_t MAX_QUEUE_SIZE = 25;
  
  struct ClientData {
    libwebsocket_context* context = 0;
    libwebsocket *wsi = 0;
    std::queue<std::string> queue; // message queue
  };

  std::map<libwebsocket*, ClientData> clients;
  // guards all access to clients 
  std::mutex clients_mutex;
};

